@@ -1,7 +1,5 @@
 package Net::Stomp::MooseHelpers::TracerRole;
-{
-  $Net::Stomp::MooseHelpers::TracerRole::VERSION = '2.4';
-}
+$Net::Stomp::MooseHelpers::TracerRole::VERSION = '2.5';
 {
   $Net::Stomp::MooseHelpers::TracerRole::DIST = 'Net-Stomp-MooseHelpers';
 }
@@ -80,6 +78,17 @@ sub _filename_from_frame {
                                 DIR => $dir->stringify);
 }
 
+sub _temp_trace_file {
+    my ($self,$frame,$direction) = @_;
+
+    my $base = sprintf '%0.5f',Time::HiRes::time();
+    my $dir = $self->trace_basedir->subdir('tmp');
+    $dir->mkpath({mode => $self->trace_directory_permissions});
+
+    return File::Temp::tempfile("${base}-temp-XXXX",
+                                DIR => $dir->stringify);
+}
+
 sub _save_frame {
     my ($self,$frame,$direction) = @_;
 
@@ -94,16 +103,21 @@ sub _save_frame {
         return;
     }
 
-    my ($fh,$filename) = $self->_filename_from_frame($frame,$direction);
-    binmode $fh;
-    syswrite $fh,$frame->as_string;
+    my ($tmp_fh,$tmp_filename) = $self->_temp_trace_file;
+    binmode $tmp_fh;
+    syswrite $tmp_fh,$frame->as_string;
     try {
-        chmod $self->trace_permissions & (~umask),$fh;
+        chmod $self->trace_permissions & (~umask),$tmp_fh;
     }
     catch {
-        chmod $self->trace_permissions & (~umask),$filename;
+        chmod $self->trace_permissions & (~umask),$tmp_filename;
     };
+    close $tmp_fh;
+
+    my ($fh,$filename) = $self->_filename_from_frame($frame,$direction);
     close $fh;
+    rename $tmp_filename,$filename;
+
     return;
 }
 
@@ -121,7 +135,7 @@ Net::Stomp::MooseHelpers::TracerRole - role to dump Net::Stomp frames to disk
 
 =head1 VERSION
 
-version 2.4
+version 2.5
 
 =head1 DESCRIPTION
 
@@ -193,7 +207,7 @@ Gianni Ceccarelli <gianni.ceccarelli@net-a-porter.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2012 by Net-a-porter.com.
+This software is copyright (c) 2014 by Net-a-porter.com.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
